from spaceone.core.pygrpc import BaseAPI
from spaceone.api.identity.v2 import authorization_pb2, authorization_pb2_grpc
from spaceone.identity.service.authorization_service import AuthorizationService


class Authorization(BaseAPI, authorization_pb2_grpc.AuthorizationServicer):
    pb2 = authorization_pb2
    pb2_grpc = authorization_pb2_grpc

    def verify(self, request, context):
        params, metadata = self.parse_request(request, context)
        authorization_svc = AuthorizationService(metadata)
        response: dict = authorization_svc.verify(params)
        return self.dict_to_message(response)



from typing import Union, List, Literal
from pydantic import BaseModel

__all__ = ["EndpointsResponse"]

State = Literal["ACTIVE", "INACTIVE"]


class EndpointResponse(BaseModel):
    name: Union[str, None] = None
    service: Union[str, None] = None
    endpoint: Union[str, None] = None
    state: Union[State, None] = None
    version: Union[str, None] = None


class EndpointsResponse(BaseModel):
    results: List[EndpointResponse]
    total_count: int



from typing import Union
from pydantic import BaseModel

__all__ = ["EndpointSearchQueryRequest"]


class EndpointSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    service: Union[str, None] = None



import logging
from typing import Union

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.identity.model.role_binding.request import *
from spaceone.identity.model.role_binding.response import *
from spaceone.identity.manager.role_binding_manager import RoleBindingManager
from spaceone.identity.manager.role_manager import RoleManager
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.manager.workspace_manager import WorkspaceManager
from spaceone.identity.error.error_role import ERROR_NOT_ALLOWED_ROLE_TYPE

_LOGGER = logging.getLogger(__name__)


class RoleBindingService(BaseService):

    service = "identity"
    resource = "RoleBinding"
    permission_group = "COMPOUND"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.role_binding_manager = RoleBindingManager()
        self.user_mgr = UserManager()

    @transaction(scope="workspace_owner:write")
    @convert_model
    def create(self, params: RoleBindingCreateRequest) -> Union[RoleBindingResponse, dict]:
        """ create role binding

         Args:
            params (RoleBindingCreateRequest): {
                'user_id': 'str',                   # required
                'role_id': 'str',                   # required
                'permission_group': 'str',          # required
                'workspace_id': 'str',
                'domain_id': 'str'                  # required
            }

        Returns:
            RoleBindingResponse:
        """

        rb_vo = self.create_role_binding(params.dict())
        return RoleBindingResponse(**rb_vo.to_dict())

    def create_role_binding(self, params: dict):
        user_id = params['user_id']
        role_id = params['role_id']
        permission_group = params['permission_group']
        domain_id = params['domain_id']
        workspace_id = params.get('workspace_id')

        # Check user
        user_vo = self.user_mgr.get_user(user_id, domain_id)

        # Check workspace
        if permission_group == 'WORKSPACE':
            if workspace_id != '*':
                workspace_mgr = WorkspaceManager()
                workspace_mgr.get_workspace(workspace_id, domain_id)
        else:
            params['workspace_id'] = '*'

        # Check role
        role_mgr = RoleManager()
        role_vo = role_mgr.get_role(role_id, domain_id)

        if permission_group == 'DOMAIN':
            if role_vo.role_type not in ['SYSTEM_ADMIN', 'DOMAIN_ADMIN']:
                raise ERROR_NOT_ALLOWED_ROLE_TYPE(supported_role_type=['DOMAIN_ADMIN'])
        else:
            if role_vo.role_type not in ['WORKSPACE_OWNER', 'WORKSPACE_MEMBER']:
                raise ERROR_NOT_ALLOWED_ROLE_TYPE(supported_role_type=['WORKSPACE_OWNER', 'WORKSPACE_MEMBER'])

        # TODO: check duplication of role_binding

        params['role_type'] = role_vo.role_type

        # Update user role type
        latest_role_type = self._get_latest_role_type(user_vo.role_type, role_vo.role_type)
        self.user_mgr.update_user_by_vo({'role_type': latest_role_type}, user_vo)

        # Create role binding
        return self.role_binding_manager.create_role_binding(params)

    @transaction(scope="workspace_owner:write")
    @convert_model
    def update_role(self, params: RoleBindingUpdateRoleRequest) -> Union[RoleBindingResponse, dict]:
        """ update role of role binding

         Args:
            params (RoleBindingUpdateRoleRequest): {
                'role_binding_id': 'str',           # required
                'role_id': 'str',                   # required
                'workspace_id': 'str',
                'domain_id': 'str',                 # required
            }

        Returns:
            RoleBindingResponse:
        """

        request_user_id = self.transaction.get_meta('user_id')

        rb_vo = self.role_binding_manager.get_role_binding(
            params.role_binding_id, params.domain_id, params.workspace_id
        )

        # TODO: check request_user_id is same with user_id of role_binding
        # TODO: check this user is last admin of domain or system

        # Check role
        role_mgr = RoleManager()
        new_role_vo = role_mgr.get_role(params.role_id, params.domain_id)

        if rb_vo.role_type in ['WORKSPACE_OWNER', 'WORKSPACE_MEMBER']:
            if new_role_vo.role_type not in ['WORKSPACE_OWNER', 'WORKSPACE_MEMBER']:
                raise ERROR_NOT_ALLOWED_ROLE_TYPE(supported_role_type=['WORKSPACE_OWNER', 'WORKSPACE_MEMBER'])
        elif rb_vo.role_type != new_role_vo.role_type:
            raise ERROR_NOT_ALLOWED_ROLE_TYPE(supported_role_type=[rb_vo.role_type])

        user_vo = self.user_mgr.get_user(rb_vo.user_id, rb_vo.domain_id)

        latest_role_type = self._get_latest_role_type(rb_vo.role_type, new_role_vo.role_type)
        self.user_mgr.update_user_by_vo({'role_type': latest_role_type}, user_vo)

        rb_vo = self.role_binding_manager.update_role_binding_by_vo(
            {
                'role_id': params.role_id,
                'role_type': new_role_vo.role_type
            },
            rb_vo
        )

        return RoleBindingResponse(**rb_vo.to_dict())

    @transaction(scope="workspace_owner:write")
    @convert_model
    def get(self, params: RoleBindingGetRequest) -> Union[RoleBindingResponse, dict]:
        """ get role binding

         Args:
            params (RoleBindingGetRequest): {
                'role_binding_id': 'str',       # required
                'workspace_id': 'str',
                'domain_id': 'str',             # required
            }

        Returns:
             RoleBindingResponse:
        """

        rb_vo = self.role_binding_manager.get_role_binding(
            params.role_binding_id, params.domain_id, params.workspace_id
        )

        return RoleBindingResponse(**rb_vo.to_dict())

    @transaction(scope="workspace_owner:write")
    @convert_model
    def delete(self, params: RoleBindingDeleteRequest) -> None:
        """ delete role binding

         Args:
            params (RoleBindingDeleteRequest): {
                'role_binding_id': 'str',       # required
                'workspace_id': 'str',
                'domain_id': 'str',             # required
            }

        Returns:
            None
        """

        request_user_id = self.transaction.get_meta('user_id')

        rb_vo = self.role_binding_manager.get_role_binding(
            params.role_binding_id, params.domain_id, params.workspace_id
        )

        # TODO: check request_user_id is same with user_id of role_binding
        # TODO: check this user is last admin of domain or system

        # Update user role type
        remain_rb_vos = self.role_binding_manager.filter_role_bindings(
            domain_id=params.domain_id,
            user_id=rb_vo.user_id
        )

        latest_role_type = 'USER'
        for remain_rb_vo in remain_rb_vos:
            if remain_rb_vo.role_binding_id == params.role_binding_id:
                continue

            latest_role_type = self._get_latest_role_type(latest_role_type, remain_rb_vo.role_type)

        user_vo = self.user_mgr.get_user(rb_vo.user_id, rb_vo.domain_id)
        self.user_mgr.update_user_by_vo({'role_type': latest_role_type}, user_vo)

        self.role_binding_manager.delete_role_binding_by_vo(rb_vo)

    @transaction(scope="workspace_owner:read")
    @append_query_filter([
        'role_binding_id', 'user_id', 'role_id', 'scope', 'workspace_id', 'domain_id'
    ])
    @append_keyword_filter(['role_binding_id', 'user_id', 'role_id'])
    @convert_model
    def list(self, params: RoleBindingSearchQueryRequest) -> Union[RoleBindingsResponse, dict]:
        """ list role bindings

        Args:
            params (RoleBindingSearchQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'role_binding_id': 'str',
                'role_type': 'str',
                'user_id': 'str',
                'role_id': 'str',
                'permission_group': 'str',
                'workspace_id': 'str',
                'domain_id': 'str',                     # required
            }

        Returns:
            RoleBindingsResponse:
        """

        query = params.query or {}
        rb_vos, total_count = self.role_binding_manager.list_role_bindings(query)

        rbs_info = [rb_vo.to_dict() for rb_vo in rb_vos]
        return RoleBindingsResponse(results=rbs_info, total_count=total_count)

    @transaction(scope="workspace_owner:read")
    @append_query_filter(['workspace_id', 'domain_id'])
    @append_keyword_filter(['role_binding_id', 'user_id', 'role_id'])
    @convert_model
    def stat(self, params: RoleBindingStatQueryRequest) -> dict:
        """ stat role bindings

        Args:
            params (RoleBindingStatQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)', # required
                'workspace_id': 'str',
                'domain_id': 'str',         # required
            }

        Returns:
            dict: {
                'results': 'list',
                'total_count': 'int'
            }
        """

        query = params.query or {}
        return self.role_binding_manager.stat_role_bindings(query)

    @staticmethod
    def _get_latest_role_type(before: str, after: str) -> str:
        priority = {
            'SYSTEM': 1,
            'SYSTEM_ADMIN': 2,
            'DOMAIN_ADMIN': 3,
            'WORKSPACE_OWNER': 4,
            'WORKSPACE_MEMBER': 5,
            'USER': 6
        }

        before_priority = priority.get(before, 6)
        after_priority = priority.get(after, 6)

        if before_priority < after_priority:
            return before
        else:
            return after



import logging

from datetime import datetime, timedelta
from typing import Union

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.identity.error.error_api_key import *
from spaceone.identity.manager.api_key_manager import APIKeyManager
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.model.api_key.request import *
from spaceone.identity.model.api_key.response import *

_LOGGER = logging.getLogger(__name__)


class APIKeyService(BaseService):
    service = "identity"
    resource = "APIKey"
    permission_group = "USER"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.api_key_mgr = APIKeyManager()

    @transaction(scope="user:write")
    @convert_model
    def create(self, params: APIKeyCreateRequest) -> Union[APIKeyResponse, dict]:
        """Create API Key
        Args:
            params (dict): {
                'user_id': 'str', # required
                'name': 'str',
                'expired_at': 'str'
                'domain_id': 'str', # required
            }
        Return:
            APIKeyResponse:
        """

        params.expired_at = self._get_expired_at(params.expired_at)
        self._check_expired_at(params.expired_at)

        user_mgr = UserManager()
        user_vo = user_mgr.get_user(params.user_id, params.domain_id)

        api_key_vo, api_key = self.api_key_mgr.create_api_key_by_user_vo(
            user_vo, params.dict()
        )

        api_key_vos = self.api_key_mgr.filter_api_keys(
            user_id=user_vo.user_id, domain_id=params.domain_id
        )

        user_mgr.update_user_by_vo({"api_key_count": api_key_vos.count()}, user_vo)
        return APIKeyResponse(**api_key_vo.to_dict(), api_key=api_key)

    @transaction(scope="user:write")
    @convert_model
    def update(self, params: APIKeyUpdateRequest) -> Union[APIKeyResponse, dict]:
        """Update API Key
        Args:
            params (dict): {
                'api_key_id': 'str',    # required
                'name': 'str',
                'domain_id': 'str',     # required
                'user_id': 'str',       # from meta
            }
        Returns:
            APIKeyResponse:
        """

        api_key_vo = self.api_key_mgr.get_api_key(
            params.api_key_id, params.domain_id, "USER", user_id=params.user_id
        )

        api_key_vo = self.api_key_mgr.update_api_key_by_vo(
            params.dict(exclude_unset=True), api_key_vo
        )

        return APIKeyResponse(**api_key_vo.to_dict())

    @transaction(scope="user:write")
    @convert_model
    def enable(self, params: APIKeyEnableRequest) -> Union[APIKeyResponse, dict]:
        """Enable API Key
        Args:
            params (dict): {
                'api_key_id': 'str',    # required
                'domain_id': 'str',     # required
                'user_id': 'str',       # from meta
            }
        Returns:
            APIKeyResponse:
        """

        api_key_vo = self.api_key_mgr.get_api_key(
            params.api_key_id, params.domain_id, "USER", user_id=params.user_id
        )

        api_key_vo = self.api_key_mgr.enable_api_key(api_key_vo)
        return APIKeyResponse(**api_key_vo.to_dict())

    @transaction(scope="user:write")
    @convert_model
    def disable(self, params: APIKeyDisableRequest) -> Union[APIKeyResponse, dict]:
        """Disable API Key
        Args:
            params (dict): {
                'api_key_id': 'str',    # required
                'domain_id': 'str',     # required
                'user_id': 'str',       # from meta
            }
        Returns:
            APIKeyResponse:
        """

        api_key_vo = self.api_key_mgr.get_api_key(
            params.api_key_id, params.domain_id, "USER", user_id=params.user_id
        )

        api_key_vo = self.api_key_mgr.disable_api_key(api_key_vo)
        return APIKeyResponse(**api_key_vo.to_dict())

    @transaction(scope="user:write")
    @convert_model
    def delete(self, params: APIKeyDeleteRequest) -> None:
        """Delete API Key
        Args:
            params (dict): {
                'api_key_id': 'str',    # required
                'domain_id': 'str',     # required
                'user_id': 'str',       # from meta
            }
        Returns:
            None
        """

        api_key_vo = self.api_key_mgr.get_api_key(
            params.api_key_id, params.domain_id, "USER", user_id=params.user_id
        )

        if api_key_vo.user_id:
            user_mgr = UserManager()
            user_vo = user_mgr.get_user(api_key_vo.user_id, params.domain_id)
            api_key_vos = self.api_key_mgr.filter_api_keys(
                user_id=user_vo.user_id, domain_id=params.domain_id
            )

            user_mgr.update_user_by_vo({"api_key_count": api_key_vos.count()}, user_vo)

        self.api_key_mgr.delete_api_key_by_vo(api_key_vo)

    @transaction(scope="user:read")
    @convert_model
    def get(self, params: APIKeyGetRequest) -> Union[APIKeyResponse, dict]:
        """Get API Key
        Args:
            params (dict): {
                'api_key_id': 'str',    # required
                'domain_id': 'str',     # required
                'user_id': 'str',       # from meta
            }
        Returns:
            APIKeyResponse:
        """

        api_key_vo = self.api_key_mgr.get_api_key(
            params.api_key_id, params.domain_id, "USER", user_id=params.user_id
        )

        return APIKeyResponse(**api_key_vo.to_dict())

    @transaction(scope="user:read")
    @append_query_filter(["api_key_id", "name", "user_id", "state", "domain_id"])
    @append_keyword_filter(["api_key_id", "name"])
    @convert_model
    def list(self, params: APIKeySearchQueryRequest) -> Union[APIKeysResponse, dict]:
        """List API Keys
        Args:
            params (dict): {
                'query': 'dict',
                'api_key_id': 'str',
                'name': 'str',
                'user_id': 'str',
                'state': 'str',
                'domain_id': 'str'      # required
            }
        Returns:
            APIKeysResponse:
        """

        query = params.query
        query = self._append_owner_type_filter(query)

        api_key_vos, total_count = self.api_key_mgr.list_api_keys(query)
        api_keys_info = [api_key_vo.to_dict() for api_key_vo in api_key_vos]
        return APIKeysResponse(results=api_keys_info, total_count=total_count)

    @transaction(scope="user:read")
    @append_query_filter(["user_id" "owner_type", "domain_id"])
    @append_keyword_filter(["api_key_id", "name"])
    @convert_model
    def stat(self, params: APIKeyStatQueryRequest) -> dict:
        """Stat API Keys
        Args:
            params (dict): {
                'query': 'dict',        # required
                'domain_id': 'str',     # required
                'user_id': 'str',       # from meta
            }
        Returns:
            dict: {
                'results': 'list',
                'total_count': 'int'
            }
        """
        query = params.query or {}
        query = self._append_owner_type_filter(query)

        return self.api_key_mgr.stat_api_keys(query)

    @staticmethod
    def _append_owner_type_filter(query: dict):
        query.get("filter", []).append({"k": "owner_type", "v": "USER", "o": "eq"})
        return query

    @staticmethod
    def _get_expired_at(expired_at: str) -> str:
        if expired_at:
            return expired_at
        else:
            return (datetime.now() + timedelta(days=365)).strftime("%Y-%m-%d %H:%M:%S")

    @staticmethod
    def _check_expired_at(expired_at: str) -> None:
        one_year_later = datetime.now() + timedelta(days=365)

        if one_year_later.strftime("%Y-%m-%d %H:%M:%S") < expired_at:
            raise ERROR_API_KEY_EXPIRED_LIMIT(expired_at=expired_at)



from datetime import datetime
from typing import List, Union, Literal
from pydantic import BaseModel

from spaceone.core import utils

from spaceone.identity.model.domain.request import State

__all__ = [
    "DomainResponse",
    "DomainsResponse",
    "DomainAuthInfoResponse",
    "DomainSecretResponse",
]

ExternalAuthState = Literal["ENABLED", "DISABLED"]


class DomainResponse(BaseModel):
    domain_id: Union[str, None] = None
    name: Union[str, None] = None
    state: Union[State, None] = None
    tags: Union[dict, None] = None
    created_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        return data


class DomainAuthInfoResponse(BaseModel):
    domain_id: Union[str, None] = None
    name: Union[str, None] = None
    external_auth_state: Union[ExternalAuthState, None] = None
    metadata: Union[dict, None] = None


class DomainSecretResponse(BaseModel):
    domain_id: Union[str, None] = None
    public_key: Union[str, None] = None


class DomainsResponse(BaseModel):
    results: List[DomainResponse]
    total_count: int



from datetime import datetime
from typing import Union
from pydantic import BaseModel

__all__ = [
    "APIKeyCreateRequest",
    "APIKeyUpdateRequest",
    "APIKeyDeleteRequest",
    "APIKeyGetRequest",
    "APIKeySearchQueryRequest",
    "APIKeyEnableRequest",
    "APIKeyDisableRequest",
    "APIKeyStatQueryRequest",
]


class APIKeyCreateRequest(BaseModel):
    user_id: str
    name: Union[str, None] = None
    expired_at: Union[str, None] = None
    domain_id: str


class APIKeyUpdateRequest(BaseModel):
    api_key_id: str
    name: Union[str, None] = None
    domain_id: str
    user_id: Union[str, None] = None


class APIKeyEnableRequest(BaseModel):
    api_key_id: str
    domain_id: str
    user_id: Union[str, None] = None


class APIKeyDisableRequest(BaseModel):
    api_key_id: str
    domain_id: str
    user_id: Union[str, None] = None


class APIKeyDeleteRequest(BaseModel):
    api_key_id: str
    domain_id: str
    user_id: Union[str, None] = None


class APIKeyGetRequest(BaseModel):
    api_key_id: str
    domain_id: str
    user_id: Union[str, None] = None


class APIKeySearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    api_key_id: Union[str, None] = None
    name: Union[str, None] = None
    user_id: Union[str, None] = None
    state: Union[str, None] = None
    domain_id: str


class APIKeyStatQueryRequest(BaseModel):
    query: dict
    domain_id: str
    user_id: Union[str, None] = None



from datetime import datetime
from typing import Union, List
from pydantic import BaseModel, Field
from spaceone.core import utils
from spaceone.identity.model.schema.request import SchemaType

__all__ = ["SchemaResponse", "SchemasResponse"]


class SchemaResponse(BaseModel):
    schema_id: Union[str, None] = None
    name: Union[str, None] = None
    schema_type: Union[SchemaType, None] = None
    data_schema: Union[dict, None] = Field(None, alias="schema")
    provider: Union[str, None] = None
    related_schemas: Union[List[str], None] = None
    options: Union[dict, None] = None
    tags: Union[dict, None] = None
    is_managed: Union[bool, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None
    updated_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)

        if 'data_schema' in data:
            data['schema'] = data['data_schema']
            del data['data_schema']

        data['created_at'] = utils.datetime_to_iso8601(data['created_at'])
        data['updated_at'] = utils.datetime_to_iso8601(data['updated_at'])
        return data


class SchemasResponse(BaseModel):
    results: List[SchemaResponse] = []
    total_count: int



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class App(MongoModel):
    app_id = StringField(max_length=40, generate_id="app", unique=True)
    name = StringField(max_length=40, required=True)
    state = StringField(
        max_length=20, default="ENABLED", choices=("ENABLED", "DISABLED", "EXPIRED")
    )
    role_type = StringField(
        max_length=20,
        default="WORKSPACE_MEMBER",
        choices=(
            "SYSTEM",
            "SYSTEM_ADMIN",
            "DOMAIN_ADMIN",
            "WORKSPACE_OWNER",
            "WORKSPACE_MEMBER",
        ),
    )
    api_key_id = StringField(max_length=40, default=None, null=True)
    role_id = StringField(max_length=40, required=True)

    domain_id = StringField(max_length=40, required=True)
    created_at = DateTimeField(auto_now_add=True)
    last_accessed_at = DateTimeField(default=None, null=True)
    expired_at = DateTimeField(required=True)

    meta = {
        "updatable_fields": ["name", "state", "api_key_id", "tags", "last_accessed_at"],
        "minimal_fields": ["app_id", "state", "expired_at", "api_key_id"],
        "ordering": ["app_id"],
        "indexes": ["state", "domain_id", "last_accessed_at", "expired_at"],
    }



from datetime import datetime
from typing import Union, Literal, List
from pydantic import BaseModel
from spaceone.core import utils

__all__ = ["ServiceAccountResponse", "ServiceAccountsResponse"]

Scope = Literal["DOMAIN", "WORKSPACE"]


class ServiceAccountResponse(BaseModel):
    service_account_id: Union[str, None] = None
    name: Union[str, None] = None
    data: Union[dict, None] = None
    provider: Union[str, None] = None
    tags: Union[dict, None] = None
    trusted_account_id: Union[str, None] = None
    project_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data['created_at'] = utils.datetime_to_iso8601(data['created_at'])
        return data


class ServiceAccountsResponse(BaseModel):
    results: List[ServiceAccountResponse]
    total_count: int



from datetime import datetime
from typing import Union, List
from pydantic import BaseModel

__all__ = ["UserGroupResponse", "UserGroupsResponse"]


class UserGroupResponse(BaseModel):
    user_group_id: Union[str, None] = None
    name: Union[str, None] = None
    users: Union[List[str], None] = None
    tags: Union[dict, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None


class UserGroupsResponse(BaseModel):
    results: List[UserGroupResponse]
    total_count: int



from spaceone.core.pygrpc import BaseAPI
from spaceone.api.identity.v2 import service_account_pb2, service_account_pb2_grpc
from spaceone.identity.service.service_account_service import (
    ServiceAccountService,
)


class ServiceAccount(BaseAPI, service_account_pb2_grpc.ServiceAccountServicer):
    pb2 = service_account_pb2
    pb2_grpc = service_account_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.create(params)
        return self.dict_to_message(response)

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.update(params)
        return self.dict_to_message(response)

    def change_trusted_account(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.change_trusted_account(params)
        return self.dict_to_message(response)

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        service_account_svc.delete(params)
        return self.empty()

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.get(params)
        return self.dict_to_message(response)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.list(params)
        return self.dict_to_message(response)

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)
        service_account_svc = ServiceAccountService(metadata)
        response: dict = service_account_svc.stat(params)
        return self.dict_to_message(response)



import logging
import random
from abc import abstractmethod, ABC, ABCMeta

from spaceone.core import config, utils, cache
from spaceone.core.manager import BaseManager

from spaceone.identity.error.error_mfa import ERROR_NOT_SUPPORTED_MFA_TYPE
from spaceone.identity.error.error_user import ERROR_INVALID_VERIFY_CODE

__all__ = ["BaseMFAManager", "MFAManager"]
_LOGGER = logging.getLogger(__name__)


class BaseMFAManager(BaseManager, ABC):
    @abstractmethod
    def enable_mfa(self, **kwargs):
        pass

    @abstractmethod
    def disable_mfa(self, **kwargs):
        pass

    def confirm_mfa(self, **kwargs):
        pass

    def _load_conf(self):
        identity_conf = config.get_global("IDENTITY") or {}
        mfa_conf = identity_conf.get("token", {})
        self.CONST_MFA_VERIFICATION_CODE_TIMEOUT = mfa_conf.get(
            "mfa_verify_code_timeout", 300
        )


class MFAManager(BaseMFAManager, metaclass=ABCMeta):
    mfa_type: str

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._load_conf()

    def enable_mfa(self, **kwargs):
        raise NotImplementedError("MFAManager.enable_mfa not implemented!")

    def disable_mfa(self, **kwargs):
        raise NotImplementedError("MFAManager.disable_mfa not implemented!")

    def confirm_mfa(self, **kwargs):
        raise NotImplementedError("MFAManager.confirm_mfa not implemented!")

    def create_mfa_verify_code(self, user_id, domain_id):
        if cache.is_set():
            verify_code = self._generate_verify_code()
            cache.delete(f"mfa-verify-code:{domain_id}:{user_id}")
            cache.set(
                f"mfa-verify-code:{domain_id}:{user_id}",
                verify_code,
                expire=self.CONST_MFA_VERIFICATION_CODE_TIMEOUT,
            )
            return verify_code

    @classmethod
    def get_manager_by_mfa_type(cls, mfa_type):
        for subclass in cls.__subclasses__():
            if subclass.mfa_type == mfa_type:
                return subclass()
        raise ERROR_NOT_SUPPORTED_MFA_TYPE(support_mfa_types=["EMAIL"])

    @staticmethod
    def check_mfa_verify_code(user_id, domain_id, verify_code):
        if cache.is_set():
            cached_verify_code = cache.get(f"mfa-verify-code:{domain_id}:{user_id}")
            if cached_verify_code == verify_code:
                cache.delete(f"mfa-verify-code:{domain_id}:{user_id}")
                return True
        raise ERROR_INVALID_VERIFY_CODE(verify_code=verify_code)

    @staticmethod
    def _generate_verify_code():
        return str(random.randint(100000, 999999))



from spaceone.core.error import *


class ERROR_RELATED_PROJECT_EXIST(ERROR_INVALID_ARGUMENT):
    _message = "Related project is exist. (project_id = {project_id})"


class ERROR_RELATED_PROJECT_GROUP_EXIST(ERROR_INVALID_ARGUMENT):
    _message = "Related project group is exist. (project_group_id = {project_group_id})"



# Root Domain Settings
ROOT_DOMAIN_NAME = "root"

# Email Settings
EMAIL_CONSOLE_DOMAIN = ""
EMAIL_SERVICE_NAME = "Cloudforet"

# Enums: ACCESS_TOKEN (default) | PASSWORD
RESET_PASSWORD_TYPE = "ACCESS_TOKEN"

# Database Settings
DATABASE_AUTO_CREATE_INDEX = True
DATABASES = {
    "default": {
        # 'db': '',
        # 'host': '',
        # 'port': 0,
        # 'username': '',
        # 'password': '',
        # 'ssl': False,
        # 'ssl_ca_certs': ''
    }
}

# Cache Settings
CACHES = {
    "default": {},
    "local": {
        "backend": "spaceone.core.cache.local_cache.LocalCache",
        "max_size": 128,
        "ttl": 300,
    },
}

# Identity Settings
IDENTITY = {
    "token": {
        "verify_code_timeout": 3600,
        "temporary_token_timeout": 86400,  # 24 hours
        "token_timeout": 1200,
        "refresh_timeout": 1800,
        "refresh_ttl": 18,
        "refresh_once": False,
    },
    "mfa": {"mfa_verify_code_timeout": 300},
}

# Handler Settings
HANDLERS = {
    # "authentication": [{
    #     "backend": "spaceone.core.handler.authentication_handler.AuthenticationGRPCHandler",
    #     "uri": "grpc://localhost:50051/v1/Domain/get_public_key"
    # }],
    # "authorization": [{
    #     "backend": "spaceone.core.handler.authorization_handler.AuthorizationGRPCHandler",
    #     "uri": "grpc://localhost:50051/v1/Authorization/verify"
    # }],
    # "mutation": [{
    #     "backend": "spaceone.core.handler.mutation_handler.SpaceONEMutationHandler"
    # }],
    # "event": []
}

# Connector Settings
CONNECTORS = {
    "SpaceConnector": {
        "backend": "spaceone.core.connector.space_connector.SpaceConnector",
        "endpoints": {
            "identity": "grpc://localhost:50051",
            "plugin": "grpc://plugin:50051",
            "secret": "grpc://secret:50051",
            "repository": "grpc://repository:50051",
        },
    },
    "SMTPConnector": {
        # "host": "smtp.mail.com",
        # "port": "1234",
        # "user": "cloudforet",
        # "password": "1234",
        # "from_email": "support@cloudforet.com",
    },
}

# Endpoint Settings
ENDPOINTS = [
    # {
    #     "service": "identity",
    #     "name": "Identity Service",
    #     "endpoint": "grpc://<endpoint>>:<port>"
    # },
    # {
    #     "service": "inventory",
    #     "name": "Inventory Service",
    #     "endpoint": "grpc+ssl://<endpoint>>:<port>"
    # }
]



import logging

from spaceone.core import cache
from spaceone.core.auth.jwt import JWTAuthenticator, JWTUtil

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.identity.error.error_authentication import *
from spaceone.identity.error.error_domain import ERROR_DOMAIN_STATE
from spaceone.identity.error.error_mfa import *
from spaceone.identity.manager.domain_manager import DomainManager
from spaceone.identity.manager.domain_secret_manager import DomainSecretManager
from spaceone.identity.manager.mfa_manager import MFAManager
from spaceone.identity.manager.token_manager.base import JWTManager
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.model.token.request import *
from spaceone.identity.model.token.response import *

_LOGGER = logging.getLogger(__name__)


class TokenService(BaseService):
    service = "identity"
    resource = "Token"
    permission_group = "PUBLIC"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain_mgr = DomainManager()
        self.domain_secret_mgr = DomainSecretManager()
        self.user_mgr = UserManager()

    @transaction(scope="public")
    @convert_model
    def issue(self, params: TokenIssueRequest) -> Union[TokenResponse, dict]:
        """Issue token
        Args:
            params (dict): {
                'credentials': 'dict', # required
                'auth_type': 'str', # required
                'timeout': 'int',
                'refresh_count': 'int',
                'verify_code': 'str',
                'domain_id': 'str', # required
            }
        Returns:
            TokenResponse:
        """

        user_id = params.credentials.get("user_id")
        domain_id = params.domain_id
        timeout = params.timeout
        refresh_count = params.refresh_count
        verify_code = params.verify_code

        private_jwk = self.domain_secret_mgr.get_domain_private_key(domain_id=domain_id)
        refresh_private_jwk = self.domain_secret_mgr.get_domain_refresh_private_key(
            domain_id=domain_id
        )

        # Check Domain state is ENABLED
        self._check_domain_state(domain_id)

        token_mgr = JWTManager.get_token_manager_by_auth_type(params.auth_type)
        token_mgr.authenticate(user_id, domain_id, params.credentials)

        user_vo = token_mgr.user
        user_mfa = user_vo.mfa.to_dict() if user_vo.mfa else {}

        if user_mfa.get("state", "DISABLED") == "ENABLED":
            mfa_manager = MFAManager.get_manager_by_mfa_type(user_mfa.get("mfa_type"))
            if verify_code:
                mfa_manager.check_mfa_verify_code(user_id, domain_id, verify_code)
            else:
                mfa_email = user_mfa["options"].get("email")
                mfa_manager.send_mfa_authentication_email(
                    user_id, domain_id, mfa_email, user_vo.language
                )
                raise ERROR_MFA_REQUIRED(user_id=mfa_email)

        token_info = token_mgr.issue_token(
            private_jwk=private_jwk,
            refresh_private_jwk=refresh_private_jwk,
            timeout=timeout,
            ttl=refresh_count,
            domain_id=domain_id,
        )

        return TokenResponse(**token_info)

    @transaction(scope="public")
    @convert_model
    def refresh(self, params: dict) -> Union[TokenResponse, dict]:
        """Refresh token
        Args:
            params (dict): {}
        Returns:
            TokenResponse:
        """
        refresh_token = self.transaction.get_meta("token")

        if refresh_token is None:
            raise ERROR_INVALID_REFRESH_TOKEN()

        domain_id = self._extract_domain_id(refresh_token)

        private_jwk = self.domain_secret_mgr.get_domain_private_key(domain_id=domain_id)
        refresh_public_jwk = self.domain_secret_mgr.get_domain_refresh_public_key(
            domain_id=domain_id
        )
        refresh_private_jwk = self.domain_secret_mgr.get_domain_refresh_private_key(
            domain_id=domain_id
        )

        token_info = self._verify_refresh_token(refresh_token, refresh_public_jwk)
        user_auth_type = self._get_user_auth_type(token_info["aud"], domain_id)

        token_mgr = JWTManager.get_token_manager_by_auth_type(user_auth_type)
        token_mgr.check_refreshable(token_info["jti"], token_info["ttl"])

        token_response = token_mgr.refresh_token(
            token_info["user_id"],
            domain_id,
            ttl=token_info["ttl"] - 1,
            private_jwk=private_jwk,
            refresh_private_jwk=refresh_private_jwk,
        )

        return TokenResponse(**token_response)

    @cache.cacheable(key="identity:user-auth-type:{domain_id}:{user_id}", expire=600)
    def _get_user_auth_type(self, user_id, domain_id):
        try:
            user_vo = self.user_mgr.get_user(user_id, domain_id)
        except Exception as e:
            _LOGGER.error(
                f'[_get_user_backend] Authentication failure: {getattr(e, "message", e)}'
            )
            raise ERROR_AUTHENTICATION_FAILURE(user_id=user_id)

        return user_vo.auth_type

    @cache.cacheable(key="identity:domain-state:{domain_id}", expire=3600)
    def _check_domain_state(self, domain_id):
        domain_vo = self.domain_mgr.get_domain(domain_id)

        if domain_vo.state != "ENABLED":
            raise ERROR_DOMAIN_STATE(domain_id=domain_vo.domain_id)

    @staticmethod
    def _extract_domain_id(token):
        try:
            decoded = JWTUtil.unverified_decode(token)
        except Exception as e:
            _LOGGER.error(f"[_extract_domain_id] {e}")
            _LOGGER.error(token)
            raise ERROR_AUTHENTICATE_FAILURE(message="Cannot decode token.")

        domain_id = decoded.get("did")

        if domain_id is None:
            raise ERROR_AUTHENTICATE_FAILURE(message="Empty domain_id provided.")

        return domain_id

    @staticmethod
    def _verify_refresh_token(token, public_jwk):
        try:
            decoded = JWTAuthenticator(public_jwk).validate(token)
        except Exception as e:
            _LOGGER.error(f"[_verify_refresh_token] {e}")
            raise ERROR_AUTHENTICATE_FAILURE(message="Token validation failed.")

        if decoded.get("typ") != "REFRESH_TOKEN":
            raise ERROR_INVALID_REFRESH_TOKEN()

        return {
            "iss": decoded["iss"],
            "typ": decoded["typ"],
            "own": decoded["own"],
            "did": decoded["did"],
            "aud": decoded["aud"],
            "jti": decoded["jti"],
            "ttl": decoded["ttl"],
            "iat": decoded["iat"],
            "exp": decoded["exp"],
        }



import logging
from typing import Union
from spaceone.core.service import BaseService, transaction, convert_model
from spaceone.identity.model.user_group.request import *
from spaceone.identity.model.user_group.response import *

_LOGGER = logging.getLogger(__name__)


class UserGroupService(BaseService):
    @transaction
    @convert_model
    def create(self, params: UserGroupCreateRequest) -> Union[UserGroupResponse, dict]:
        return {}

    @transaction
    @convert_model
    def update(self, params: UserGroupUpdateRequest) -> Union[UserGroupResponse, dict]:
        return {}

    @transaction
    @convert_model
    def delete(self, params: UserGroupDeleteRequest) -> None:
        pass

    @transaction
    @convert_model
    def add_users(
        self, params: UserGroupAddUsersRequest
    ) -> Union[UserGroupResponse, dict]:
        return {}

    @transaction
    @convert_model
    def remove_users(
        self, params: UserGroupRemoveUsersRequest
    ) -> Union[UserGroupResponse, dict]:
        return {}

    @transaction
    @convert_model
    def get(self, params: UserGroupGetRequest) -> Union[UserGroupResponse, dict]:
        return {}

    @transaction
    @convert_model
    def list(
        self, params: UserGroupSearchQueryRequest
    ) -> Union[UserGroupsResponse, dict]:
        return {}

    @transaction
    @convert_model
    def stat(self, params: UserGroupStatQueryRequest) -> dict:
        return {}



import logging
from typing import Tuple
from mongoengine import QuerySet

from spaceone.core import utils, cache
from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.identity.model.role.database import Role
from spaceone.identity.manager.role_binding_manager import RoleBindingManager
from spaceone.identity.manager.managed_resource_manager import ManagedResourceManager
from spaceone.identity.error.error_role import ERROR_ROLE_IN_USED

_LOGGER = logging.getLogger(__name__)


class RoleManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.role_model = Role

    def create_role(self, params: dict) -> Role:
        def _rollback(vo: Role):
            _LOGGER.info(f'[create_role._rollback] Delete role: {vo.name} ({vo.role_id})')
            vo.delete()

        if 'role_id' not in params:
            params['role_id'] = utils.generate_id('role')

        if api_permissions := params.get('api_permissions'):
            params['api_permissions'] = list(set(api_permissions))

        role_vo = self.role_model.create(params)
        self.transaction.add_rollback(_rollback, role_vo)

        return role_vo

    def update_role_by_vo(
            self, params: dict, role_vo: Role
    ) -> Role:
        def _rollback(old_data):
            _LOGGER.info(f'[update_role_by_vo._rollback] Revert Data: {old_data["role_id"]}')
            role_vo.update(old_data)

        if api_permissions := params.get('api_permissions'):
            params['api_permissions'] = list(set(api_permissions))

        self.transaction.add_rollback(_rollback, role_vo.to_dict())

        return role_vo.update(params)

    @staticmethod
    def delete_role_by_vo(role_vo: Role) -> None:
        rb_mgr = RoleBindingManager()
        rb_vos = rb_mgr.filter_role_bindings(role_id=role_vo.role_id, domain_id=role_vo.domain_id)

        for rb_vo in rb_vos:
            raise ERROR_ROLE_IN_USED(role_binding_id=rb_vo.role_binding_id, user_id=rb_vo.user_id)

        role_vo.delete()

    def get_role(self, role_id: str, domain_id: str) -> Role:
        return self.role_model.get(role_id=role_id, domain_id=domain_id)

    def filter_roles(self, **conditions) -> QuerySet:
        return self.role_model.filter(**conditions)

    def list_roles(self, query: dict, domain_id: str) -> Tuple[QuerySet, int]:
        self._create_managed_role(domain_id)
        return self.role_model.query(**query)

    def stat_roles(self, query: dict) -> dict:
        return self.role_model.stat(**query)

    @cache.cacheable(key='identity:managed-role:{domain_id}:sync', expire=300)
    def _create_managed_role(self, domain_id: str) -> bool:
        managed_resource_mgr = ManagedResourceManager()

        role_vos = self.filter_roles(domain_id=domain_id, is_managed=True)

        installed_role_version_map = {}
        for role_vo in role_vos:
            installed_role_version_map[role_vo.role_id] = role_vo.version

        managed_role_map = managed_resource_mgr.get_managed_roles()

        for managed_role_id, managed_role_info in managed_role_map.items():
            managed_role_info['domain_id'] = domain_id
            managed_role_info['is_managed'] = True

            if role_version := installed_role_version_map.get(managed_role_id):
                if role_version != managed_role_info['version']:
                    _LOGGER.debug(f'[_create_managed_role] update managed role: {managed_role_id}')
                    role_vo = self.get_role(managed_role_id, domain_id)
                    self.update_role_by_vo(managed_role_info, role_vo)
            else:
                _LOGGER.debug(f'[_create_managed_role] create new managed role: {managed_role_id}')
                self.create_role(managed_role_info)

        return True



from typing import Union, Literal
from pydantic import BaseModel

__all__: ["TokenIssueRequest"]

AuthType = Literal["LOCAL", "EXTERNAL"]


class TokenIssueRequest(BaseModel):
    credentials: dict
    auth_type: AuthType
    timeout: Union[int, None] = None
    refresh_count: Union[int, None] = None
    verify_code: Union[str, None] = None
    domain_id: str



from google.protobuf.json_format import ParseDict
from spaceone.core.pygrpc import BaseAPI
from spaceone.api.identity.v2 import user_pb2, user_pb2_grpc, workspace_pb2
from spaceone.identity.service.user_service import UserService


class User(BaseAPI, user_pb2_grpc.UserServicer):
    pb2 = user_pb2
    pb2_grpc = user_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        response: dict = user_svc.create(params)
        return self.dict_to_message(response)

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        response: dict = user_svc.update(params)
        return self.dict_to_message(response)

    def verify_email(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        user_svc.verify_email(params)
        return self.empty()

    def confirm_email(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        response: dict = user_svc.confirm_email(params)
        return self.dict_to_message(response)

    def reset_password(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        user_svc.reset_password(params)
        return self.empty()

    def set_required_actions(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        response: dict = user_svc.set_required_actions(params)
        return self.dict_to_message(response)

    def enable_mfa(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        response: dict = user_svc.enable_mfa(params)
        return self.dict_to_message(response)

    def disable_mfa(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        response: dict = user_svc.disable_mfa(params)
        return self.dict_to_message(response)

    def confirm_mfa(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        response: dict = user_svc.confirm_mfa(params)
        return self.dict_to_message(response)

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        user_svc.delete(params)
        return self.empty()

    def enable(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        response: dict = user_svc.enable(params)
        return self.dict_to_message(response)

    def disable(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        response: dict = user_svc.disable(params)
        return self.dict_to_message(response)

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        response: dict = user_svc.get(params)
        return self.dict_to_message(response)

    def get_workspaces(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        response: dict = user_svc.get_workspaces(params)
        return ParseDict(response, workspace_pb2.WorkspacesInfo())

    def find(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        response: dict = user_svc.find(params)
        return self.dict_to_message(response)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        response: dict = user_svc.list(params)
        return self.dict_to_message(response)

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_svc = UserService(metadata)
        response: dict = user_svc.stat(params)
        return self.dict_to_message(response)



from typing import Union, Literal
from pydantic import BaseModel

__all__ = [
    "RoleBindingCreateRequest",
    "RoleBindingUpdateRoleRequest",
    "RoleBindingDeleteRequest",
    "RoleBindingGetRequest",
    "RoleBindingSearchQueryRequest",
    "RoleBindingStatQueryRequest",
    "PermissionGroup",
]

PermissionGroup = Literal["DOMAIN", "WORKSPACE"]


class RoleBindingCreateRequest(BaseModel):
    user_id: str
    role_id: str
    permission_group: PermissionGroup
    workspace_id: Union[str, None] = None
    domain_id: str


class RoleBindingUpdateRoleRequest(BaseModel):
    role_binding_id: str
    role_id: str
    role_type: Union[str, None] = None
    workspace_id: Union[str, None] = None
    domain_id: str


class RoleBindingDeleteRequest(BaseModel):
    role_binding_id: str
    workspace_id: Union[str, None] = None
    domain_id: str


class RoleBindingGetRequest(BaseModel):
    role_binding_id: str
    workspace_id: Union[str, None] = None
    domain_id: str


class RoleBindingSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    role_binding_id: Union[str, None] = None
    role_type: Union[str, None] = None
    permission_group: Union[PermissionGroup, None] = None
    user_id: Union[str, None] = None
    role_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    domain_id: str


class RoleBindingStatQueryRequest(BaseModel):
    query: dict
    workspace_id: Union[str, None] = None
    domain_id: str



import logging
from typing import Tuple, List

from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.identity.model.trusted_account.database import TrustedAccount

_LOGGER = logging.getLogger(__name__)


class TrustedAccountManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.trusted_account_model = TrustedAccount

    def create_trusted_account(self, params: dict) -> TrustedAccount:
        def _rollback(vo: TrustedAccount):
            _LOGGER.info(f'[create_trusted_account._rollback] '
                         f'Delete trusted service account: {vo.name} ({vo.trusted_account_id})')
            vo.delete()

        trusted_account_vo = self.trusted_account_model.create(params)
        self.transaction.add_rollback(_rollback, trusted_account_vo)

        return trusted_account_vo

    def update_trusted_account_by_vo(
        self, params: dict, trusted_account_vo: TrustedAccount
    ) -> TrustedAccount:
        def _rollback(old_data):
            _LOGGER.info(f'[update_trusted_account_by_vo._rollback] Revert Data : '
                         f'{old_data["trusted_account_id"]}')
            trusted_account_vo.update(old_data)

        self.transaction.add_rollback(_rollback, trusted_account_vo.to_dict())

        return trusted_account_vo.update(params)

    @staticmethod
    def delete_trusted_account_by_vo(trusted_account_vo: TrustedAccount) -> None:
        trusted_account_vo.delete()

    def get_trusted_account(
        self, trusted_account_id: str, domain_id: str, workspace_id: str = None
    ) -> TrustedAccount:

        conditions = {
            'trusted_account_id': trusted_account_id,
            'domain_id': domain_id,
        }

        if workspace_id:
            conditions['workspace_id'] = workspace_id

        return self.trusted_account_model.get(**conditions)

    def filter_trusted_accounts(self, **conditions) -> List[TrustedAccount]:
        return self.trusted_account_model.filter(**conditions)

    def list_trusted_accounts(self, query: dict) -> Tuple[list, int]:
        return self.trusted_account_model.query(**query)

    def stat_trusted_accounts(self, query: dict) -> dict:
        return self.trusted_account_model.stat(**query)

    def delete_trusted_secrets(
        self, trusted_account_id: str, workspace_id: str, domain_id: str
    ) -> None:
        secret_connector = SpaceConnector(service='secret')
        response = self._list_trusted_secrets(
            secret_connector,
            trusted_account_id,
            workspace_id,
            domain_id
        )

        for secret_info in response.get('results', []):
            secret_connector.dispatch('TrustedSecret.delete', {
                'trusted_secret_id': secret_info['trusted_secret_id'],
                'domain_id': domain_id,
                'workspace_id': workspace_id
            })

    @staticmethod
    def _list_trusted_secrets(
        secret_connector: SpaceConnector, trusted_account_id: str, domain_id: str, workspace_id: str
    ) -> dict:
        return secret_connector.dispatch('TrustedSecret.list', {
            'trusted_account_id': trusted_account_id,
            'domain_id': domain_id,
            'workspace_id': workspace_id
        })



from typing import Union, List
from pydantic import BaseModel

from datetime import datetime
from spaceone.core import utils

__all__ = ["APIKeyResponse", "APIKeysResponse"]


class APIKeyResponse(BaseModel):
    api_key_id: Union[str, None] = None
    api_key: Union[str, None] = None
    name: Union[str, None] = None
    state: Union[str, None] = None
    user_id: Union[str, None] = None
    domain_id: Union[str, None] = None
    created_at: Union[datetime, None] = None
    last_accessed_at: Union[datetime, None] = None
    expired_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        data["last_accessed_at"] = utils.datetime_to_iso8601(data["last_accessed_at"])
        data["expired_at"] = utils.datetime_to_iso8601(data["expired_at"])
        return data


class APIKeysResponse(BaseModel):
    results: List[APIKeyResponse]
    total_count: int
